-- Use // mod.db.profile // to interact with other options within the module
-- Use // HideBlizzard.db.profile.modules // to interact with main options

local mod = HideBlizzard:NewModule('ActionBar', 'AceEvent-3.0')
mod.modName = 'Action Bar'

local options = {
	gryphons = {
		type = 'toggle',
		name = 'Gryphons',
		get = function() return mod.db.profile.gryphons end,
		set = function(info, v)
			mod.db.profile.gryphons = v
			if v then
				mod:EnableGryphons()
			else
				mod:DisableGryphons()
			end
		end,
		disabled = function() return not HideBlizzard.db.profile.modules.ActionBar or mod.db.profile.mainmenubar end,
	},
	mainmenubar = {
		type = 'toggle',
		name = 'Main Menu Bar',
		get = function() return mod.db.profile.mainmenubar end,
		set = function(info, v)
			mod.db.profile.mainmenubar = v
			if v then
				mod:EnableMainMenuBar()
			else
				mod:DisableMainMenuBar()
			end
		end,
		disabled = function() return not HideBlizzard.db.profile.modules.ActionBar end,
	},
	repbar = {
		type = 'toggle',
		name = 'Rep bar',
		get = function() return mod.db.profile.repbar end,
		set = function(info, v)
			mod.db.profile.repbar = v
			if v then
				mod:EnableRepBar()
			else
				mod:DisableRepBar()
			end
		end,
		disabled = function() return not HideBlizzard.db.profile.modules.ActionBar or mod.db.profile.mainmenubar end,
	},
	stancebar = {
		type = 'toggle',
		name = 'Stance bar',
		get = function() return mod.db.profile.stancebar end,
		set = function(info, v)
			mod.db.profile.stancebar = v
			if v then
				mod:EnableStanceBar()
			else
				mod:DisableStanceBar()
			end
		end,
		disabled = function() return not HideBlizzard.db.profile.modules.ActionBar or mod.db.profile.mainmenubar or GetNumShapeshiftForms() == 0 end,
	},
	xpbar = {
		type = 'toggle',
		name = 'XP bar',
		get = function() return mod.db.profile.xpbar end,
		set = function(info, v)
			mod.db.profile.xpbar = v
			if v then
				mod:EnableXPBar()
			else
				mod:DisableXPBar()
			end
		end,
		disabled = function() return not HideBlizzard.db.profile.modules.ActionBar or mod.db.profile.mainmenubar end,
	},
}

local defaults = {
	profile = {
		gryphons = false,
		mainmenubar = false,
		repbar = false,
		stancebar= false,
		xpbar = false,
	},
}

function mod:OnInitialize()
	self.db = HideBlizzard.db:RegisterNamespace('ActionBar', defaults)
end

function mod:OnEnable()
	self:UpdateFrames()
end

function mod:OnDisable()
	self:DisableGryphons()
	self:DisableMainMenuBar()
	self:DisableRepBar()
	self:DisableStanceBar()
	self:DisableXPBar()
end

function mod:UpdateFrames()
	if mod.db.profile.gryphons == true then
		self:EnableGryphons()
	else
		self:DisableGryphons()
	end
	if mod.db.profile.mainmenubar == true then
		self:EnableMainMenuBar()
	else
		self:DisableMainMenuBar()
	end
	if mod.db.profile.repbar == true then
		self:EnableRepBar()
	else
		self:DisableRepBar()
	end
	if mod.db.profile.stancebar == true then
		self:EnableStanceBar()
	else
		self:DisableStanceBar()
	end
	if mod.db.profile.xpbar == true then
		self:EnableXPBar()
	else
		self:DisableXPBar()
	end
end

function mod:EnableGryphons()
	MainMenuBarLeftEndCap:Hide()
	MainMenuBarRightEndCap:Hide()
end

function mod:DisableGryphons()
	MainMenuBarLeftEndCap:Show()
	MainMenuBarRightEndCap:Show()
end

function mod:EnableMainMenuBar()
	MainMenuBar:UnregisterAllEvents()
	MainMenuBar:Hide()
end

function mod:DisableMainMenuBar()
	MainMenuBar:Show()
	MainMenuBar:RegisterEvent('PLAYER_ENTERING_WORLD')
	MainMenuBar:RegisterEvent('ACTIONBAR_PAGE_CHANGED')
	MainMenuBar:RegisterEvent('ADDON_LOADED')
	MainMenuBar:RegisterEvent('UNIT_ENTERING_VEHICLE')
	MainMenuBar:RegisterEvent('UNIT_ENTERED_VEHICLE')
	MainMenuBar:RegisterEvent('UNIT_EXITING_VEHICLE')
	MainMenuBar:RegisterEvent('UNIT_EXITED_VEHICLE')
end

function mod:EnableRepBar()
	for factionIndex = 1, GetNumFactions() do
		local isWatched = GetFactionInfo(factionIndex)
		if isWatched then
			RegisterStateDriver(ReputationWatchBar, 'visibility', 'hide')
		else
			RegisterStateDriver(ReputationWatchBar, 'visibility', 'hide')
		end
	end
end

function mod:DisableRepBar()
	for factionIndex = 1, GetNumFactions() do
		local isWatched = GetFactionInfo(factionIndex)
		if isWatched then
			RegisterStateDriver(ReputationWatchBar, 'visibility', 'show')
			SecureStateDriverManager:SetAttribute('delframe', ReputationWatchBar)
			ReputationWatchBar_Update()
		else
			RegisterStateDriver(ReputationWatchBar, 'visibility', 'hide')
		end
	end
end

function mod:EnableStanceBar()
	if GetNumShapeshiftForms() ~= 0 then
		RegisterStateDriver(ShapeshiftBarFrame, 'visibility', 'hide')
	end
end

function mod:DisableStanceBar()
	if GetNumShapeshiftForms() ~= 0 then
		RegisterStateDriver(ShapeshiftBarFrame, 'visibility', 'show')
		SecureStateDriverManager:SetAttribute('delframe', ShapeshiftBarFrame)
	else
		RegisterStateDriver(ShapeshiftBarFrame, 'visibility', 'hide')
	end
end

function mod:EnableXPBar()
	if UnitLevel('player') == MAX_PLAYER_LEVEL then
		RegisterStateDriver(MainMenuExpBar, 'visibility', 'hide')
	else
		RegisterStateDriver(MainMenuExpBar, 'visibility', 'hide')
	end
end

function mod:DisableXPBar()
	if UnitLevel('player') == MAX_PLAYER_LEVEL then
		RegisterStateDriver(MainMenuExpBar, 'visibility', 'show')
		SecureStateDriverManager:SetAttribute('delframe', MainMenuExpBar)
		MainMenuExpBar_Update()
	else
		RegisterStateDriver(MainMenuExpBar, 'visibility', 'show')
		SecureStateDriverManager:SetAttribute('delframe', MainMenuExpBar)
		MainMenuExpBar_Update()
	end
end

function mod:Info()
	return 'Hides frames connected to the main action bar.'
end

function mod:GetOptions()
	return options
end